
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include "display.h"
#include <math.h>

#define pi 3.141592653589

/* off screen buffer */
color_type buffer[display_x * display_y];
BITMAPINFO buffer_info;

int display_info = 0;
double display_time;

/* starting parameters */
double start_x = -2, start_y = -1;
double end_x = 1, end_y = 1;
int max_it = 200;


/* map the interation count to a color, and set the pixel */
void inline color_map(color_type *p, unsigned int it)
{
    double ratio = it / 255.0;

    if (it < 192) 
        p->r = ( unsigned char )(128 * sin(ratio * 2 * pi)) + 128;

    p->g = ( unsigned char )(128* sin(ratio * 2 * pi  - pi/2.0)) + 128;

    if (it > 64)
         p->b = ( unsigned char )(128 * sin(ratio * 2 * pi - pi) )+128;

}
/* end of color map */    



void draw_fractal( void )
{
    LARGE_INTEGER start_count, end_count;

    QueryPerformanceCounter(&start_count);
    
    clear_display();
   
    // variables
    double x, y = end_y; // draw upside down
    double dx = (end_x - start_x) / ((double)display_x-1);
    double dy = (end_y - start_y) / ((double)display_y-1);

    color_type *index = buffer; // pointer to start of the display buffer
    
    // each row
    for (int sy = 0; sy < display_y; sy++)
    {
        x = start_x; // reset to next row
        
        // each col
        for (int sx = 0; sx < display_x; sx++)
        {
            // preform an iteration

            unsigned int it = max_it;  // reset interation counter
            double next_zx;            // temporary storage for the old value
            double zx = 0, zy = 0;     // real and imm componets
            
            // core orbit loop
            while (it-- && (zx * zx + zy*zy) < 4)
            {
                next_zx = zx * zx - zy * zy + x;
                zy = 2*zy*zx + y;
                zx = next_zx;
            }   

            // draw if divergent           
            if (it < max_it)
                color_map(index, it);

            index++; // next buffer pixel
            x += dx;    // next point in complex plain
          
        }
        y -= dy;   // next point in complex plain
    }    

    // force redraw
    InvalidateRect(main_win, NULL, FALSE);
    
    QueryPerformanceCounter(&end_count);
    
    display_time = (double)(end_count.LowPart - start_count.LowPart) / (double)timer_freq.LowPart;
}


/* clear the buffer to black */
void clear_display( void )
{
    memset(buffer, 0, display_x * display_y * 4);
}
/* end of clear display */


/* setup the parameters for the off screen buffer container */
void create_buffer( void )
{
	// Set the new Bitmap
	buffer_info.bmiHeader.biSize = sizeof(buffer_info.bmiHeader);
	buffer_info.bmiHeader.biWidth = display_x;
	buffer_info.bmiHeader.biHeight = display_y;
	buffer_info.bmiHeader.biPlanes = 1;
	buffer_info.bmiHeader.biBitCount = 32;
	buffer_info.bmiHeader.biCompression = BI_RGB;
	buffer_info.bmiHeader.biSizeImage = display_x * display_y * 4;
	buffer_info.bmiHeader.biClrUsed = 0;
	buffer_info.bmiHeader.biClrImportant = 0;
}
/* end of create buffer */


/* uncreate the buffer */
void destroy_buffer( void )
{
    //DeleteObject(buffer);    
}
/* end of destroy_buffer */


/* show the text buffer on the screen */
void display(void)
{
    BITMAP bm;
    PAINTSTRUCT ps;

    HDC hdc = BeginPaint(main_win, &ps);
    HDC hdcMem = CreateCompatibleDC(hdc);

    HBITMAP  buffer_bitmap = CreateCompatibleBitmap(hdc, display_x, display_y);
    HBITMAP hbmOld = (HBITMAP ) SelectObject(hdcMem, buffer_bitmap);

    GetObject(buffer_bitmap, sizeof(bm), &bm);

   // set the date, remember it's flipped
	SetDIBits(hdcMem, buffer_bitmap, 0, display_y, buffer, &buffer_info, DIB_RGB_COLORS);

    if (display_info)
    {
        char b[200];
        sprintf(b, "Render: %f sec", display_time);
        TextOut(hdcMem, 0, 0, b, strlen(b));
    }

    // copy double buffer
    BitBlt(hdc, 0, 0, display_x, display_y, hdcMem, 0, 0, SRCCOPY);



    if (mouse_is_down)
    {
//        HPEN hPen = CreatePen(PS_DASHDOTDOT, 1, RGB(255, 25, 5));
//        SelectObject(hdc, hPen);
//        Rectangle(hdc, mouse_x, mouse_y, mouse_start_x, mouse_start_y);

 //HBRUSH br = CreateBrush(PS_DASHDOTDOT, 1, RGB(255, 25, 5));


    RECT r;

    r.top = mouse_y;  r.bottom = mouse_start_y;
    r.left = mouse_x;  r.right = mouse_start_x;

    int tmp;
    
    if (r.top > r.bottom)
    {
        tmp = r.top;
        r.top = r.bottom;
        r.bottom = tmp;
    }

    if (r.left > r.right)
    {
        tmp = r.right;
        r.right = r.left;
        r.left = tmp;
    }
    
    FrameRect(hdc, &r,(HBRUSH)(COLOR_BTNFACE+1));
    }


    SelectObject(hdcMem, hbmOld);

    DeleteObject(buffer_bitmap);  
    DeleteObject(hbmOld);
    
    DeleteDC(hdcMem);
    DeleteDC(hdc);

    EndPaint(main_win, &ps);
}
/* end of display_text_area */


